@@ -25,6 +25,7 @@ import("core.project.rule")
 import("core.project.config")
 import("core.project.project")
 import("core.base.bit")
+import("private.detect.check_targetname")
 
 -- get library deps
 function _get_librarydeps(target)
@@ -153,7 +154,7 @@ function main()
     -- package the given target?
     local targetname = option.get("target")
     if targetname then
-        local target = project.target(targetname)
+        local target = assert(check_targetname(targetname))
         _package_targets(target:orderdeps())
         _package_target(target)
     else
