@@ -318,7 +318,7 @@ function _initialize_shell()
         local profile_fish = "$XMAKE_PROGRAM_DIR/scripts/profile-unix.fish"
         local bridge_command = format([[export XMAKE_ROOTDIR="%s"
 export XMAKE_PROGRAM_DIR="%s"
-export PATH="$XMAKE_ROOTDIR:$PATH"
+# export PATH="$XMAKE_ROOTDIR:$PATH"
 test $FISH_VERSION && test -f "%s" && source "%s" && exit 0
 test -f "%s" && source "%s"
 ]], path.directory(os.programfile()), os.programdir(), profile_fish, profile_fish, profile, profile)
@@ -484,7 +484,7 @@ function main()
                             http.download(url, installerfile)
                         end
                     else
-                        git.clone(url, {depth = 1, recurse_submodules = not script_only, branch = version, outputdir = sourcedir})
+                        git.clone(url, {depth = 1, shallow_submodules = true, recurse_submodules = not script_only, branch = version, outputdir = sourcedir})
                     end
                     return true
                 end,
