@@ -18,6 +18,19 @@
 -- @file        is_config.lua
 --
 
--- return module
-return require("project/config").is_value
+local config  = require("project/config")
+local sandbox = require("sandbox/sandbox")
+
+return function (name, ...)
+    local namespace
+    local instance = sandbox.instance()
+    if instance then
+        namespace = instance:namespace()
+    end
+    local value = config.get(name)
+    if value == nil and namespace then
+        value = config.get(namespace .. "::" .. name)
+    end
+    return config._is_value(value, ...)
+end
 
