@@ -454,7 +454,11 @@ function _instance:_description(toolkind)
             nc         = "the nim compiler",
             ncld       = "the nim linker",
             ncsh       = "the nim shared library linker",
-            ncar       = "the nim static library archiver"
+            ncar       = "the nim static library archiver",
+            kc         = "the kotlin native compiler",
+            kcld       = "the kotlin native linker",
+            kcsh       = "the kotlin native shared library linker",
+            kcar       = "the kotlin native static library archiver",
         }
         self._DESCRIPTIONS = descriptions
     end
