@@ -21,6 +21,7 @@
 -- imports
 import("core.tool.compiler")
 import("lib.detect.find_program")
+import("lib.detect.find_programver")
 
 -- find ar
 --
@@ -37,7 +38,7 @@ import("lib.detect.find_program")
 --
 function main(opt)
     opt = opt or {}
-    local program = find_program(opt.program or "lld.link", opt)
+    local program = find_program(opt.program or "lld-link", opt)
     local version = nil
     if program and opt and opt.version then
         version = find_programver(program, opt)
