@@ -31,18 +31,32 @@ import("compiler_support")
 import(".builder", {inherit = true})
 
 function _compile_one_step(target, bmifile, sourcefile, objectfile, opt)
-    -- get flags
     local module_outputflag = compiler_support.get_moduleoutputflag(target)
-    if module_outputflag then
-        local flags = table.join({"-x", "c++-module", module_outputflag .. bmifile}, opt.std and {"-Wno-include-angled-in-module-purview", "-Wno-reserved-module-identifier"} or {})
-        if opt and opt.batchcmds then
-            _batchcmds_compile(opt.batchcmds, target, flags, sourcefile, objectfile)
+    if opt.is_mapped_bmi then
+        -- get flags
+        if module_outputflag then
+            local flags = table.join(opt.std and {"-Wno-include-angled-in-module-purview", "-Wno-reserved-module-identifier"} or {})
+            if opt and opt.batchcmds then
+                _batchcmds_compile(opt.batchcmds, target, flags, bmifile, objectfile)
+            else
+                _compile(target, flags, bmifile, objectfile)
+            end
         else
-            _compile(target, flags, sourcefile, objectfile)
+            _compile_objectfile_step(target, bmifile, sourcefile, objectfile, opt)
         end
     else
-        _compile_bmi_step(target, bmifile, sourcefile, opt)
-        _compile_objectfile_step(target, bmifile, sourcefile, objectfile, opt)
+        -- get flags
+        if module_outputflag then
+            local flags = table.join({"-x", "c++-module", module_outputflag .. bmifile}, opt.std and {"-Wno-include-angled-in-module-purview", "-Wno-reserved-module-identifier"} or {})
+            if opt and opt.batchcmds then
+                _batchcmds_compile(opt.batchcmds, target, flags, sourcefile, objectfile)
+            else
+                _compile(target, flags, sourcefile, objectfile)
+            end
+        else
+            _compile_bmi_step(target, bmifile, sourcefile, opt)
+            _compile_objectfile_step(target, bmifile, sourcefile, objectfile, opt)
+        end
     end
 end
 
@@ -122,12 +136,11 @@ function _get_requiresflags(target, module, opt)
     local name = module.name
     local cachekey = target:name() .. name
 
+    local requires, requires_changed = is_dependencies_changed(target, module)
     local requiresflags = compiler_support.memcache():get2(cachekey, "requiresflags")
-                       or compiler_support.localcache():get2(cachekey, "requiresflags")
-
-    if not requiresflags or (opt and opt.regenerate) then
+    if not requiresflags or requires_changed then
         requiresflags = {}
-        for required, _ in table.orderpairs(module.requires) do
+        for required in requires:orderitems() do
             local dep_module = get_from_target_mapper(target, required)
             assert(dep_module, "module dependency %s required for %s not found", required, name)
 
@@ -146,15 +159,16 @@ function _get_requiresflags(target, module, opt)
                 table.join2(requiresflags, deps)
             end
         end
-        compiler_support.memcache():set2(cachekey, "requiresflags", table.unique(requiresflags))
-        compiler_support.localcache():set2(cachekey, "requiresflags", table.unique(requiresflags))
+        requiresflags = table.unique(requiresflags)
+        compiler_support.memcache():set2(cachekey, "requiresflags", requiresflags)
+        compiler_support.memcache():set2(cachekey, "oldrequires", requires)
     end
     return requiresflags
 end
 
 function _append_requires_flags(target, module, name, cppfile, bmifile, opt)
     local cxxflags = {}
-    local requiresflags = _get_requiresflags(target, {name = (name or cppfile), bmi = bmifile, requires = module.requires}, {regenerate = opt.build})
+    local requiresflags = _get_requiresflags(target, {name = (name or cppfile), bmi = bmifile, requires = module.requires})
     for _, flag in ipairs(requiresflags) do
         -- we need to wrap flag to support flag with space
         if type(flag) == "string" and flag:find(" ", 1, true) then
@@ -204,14 +218,10 @@ function make_module_buildjobs(target, batchjobs, job_name, deps, opt)
         name = job_name,
         deps = table.join(target:name() .. "_populate_module_map", deps),
         sourcefile = opt.cppfile,
-        job = batchjobs:newjob(name or opt.cppfile, function(index, total)
+        job = batchjobs:newjob(name or opt.cppfile, function(index, total, jobopt)
             local mapped_bmi
             if provide and compiler_support.memcache():get2(target:name() .. name, "reuse") then
-                if not target:is_binary() then
-                    return
-                else
-                    mapped_bmi = get_from_target_mapper(target, name).bmi
-                end
+                mapped_bmi = get_from_target_mapper(target, name).bmi
             end
 
             local build, dependinfo
@@ -248,23 +258,17 @@ function make_module_buildjobs(target, batchjobs, job_name, deps, opt)
                     local fileconfig = target:fileconfig(opt.cppfile)
                     local public = fileconfig and fileconfig.public
                     local external = fileconfig and fileconfig.external
+                    local from_moduleonly = external and external.moduleonly
                     local bmifile = mapped_bmi or bmifile
-                    if target:is_binary() then
-                        if mapped_bmi then
-                            progress.show((index * 100) / total, "${color.build.target}<%s> ${clear}${color.build.object}compiling.objectfile.$(mode) %s", target:name(), name or opt.cppfile)
-                            _compile_objectfile_step(target, bmifile, opt.cppfile, opt.objectfile)
-                        else
-                            progress.show((index * 100) / total, "${color.build.target}<%s> ${clear}${color.build.object}compiling.module.$(mode) %s", target:name(), name or opt.cppfile)
-                            _compile_one_step(target, bmifile, opt.cppfile, opt.objectfile, {std = (name == "std" or name == "std.compat")})
-                        end
-                    else
-                        if not public and not external then
-                            progress.show((index * 100) / total, "${color.build.target}<%s> ${clear}${color.build.object}compiling.module.$(mode) %s", target:name(), name or opt.cppfile)
-                            _compile_one_step(target, bmifile, opt.cppfile, opt.objectfile, {std = (name == "std" or name == "std.compat")})
-                        else
-                            progress.show((index * 100) / total, "${color.build.target}<%s> ${clear}${color.build.object}compiling.bmi.$(mode) %s", target:name(), name or opt.cppfile)
+                    local is_mapped_bmi = mapped_bmi ~= nil
+                    if external and not from_moduleonly then
+                        if not mapped_bmi then
+                            progress.show(jobopt.progress, "${color.build.target}<%s> ${clear}${color.build.object}compiling.bmi.$(mode) %s", target:name(), name or opt.cppfile)
                             _compile_bmi_step(target, bmifile, opt.cppfile, {std = (name == "std" or name == "std.compat")})
                         end
+                    else
+                        progress.show(jobopt.progress, "${color.build.target}<%s> ${clear}${color.build.object}compiling.module.$(mode) %s", target:name(), name or opt.cppfile)
+                        _compile_one_step(target, bmifile, opt.cppfile, opt.objectfile, {std = (name == "std" or name == "std.compat"), is_mapped_bmi = is_mapped_bmi})
                     end
                 else
                     os.tryrm(opt.objectfile) -- force rebuild for .cpp files
@@ -282,21 +286,7 @@ function make_module_buildcmds(target, batchcmds, opt)
 
     local mapped_bmi
     if provide and compiler_support.memcache():get2(target:name() .. name, "reuse") then
-        if not target:is_binary() then
-            return
-        else
-            mapped_bmi = get_from_target_mapper(target, name).bmi
-        end
-    end
-
-    local build
-    if provide or compiler_support.has_module_extension(opt.cppfile) then
-        build = should_build(target, opt.cppfile, bmifile, {name = name, objectfile = opt.objectfile, requires = opt.module.requires})
-
-        -- needed to detect rebuild of dependencies
-        if provide and build then
-            mark_build(target, name)
-        end
+        mapped_bmi = get_from_target_mapper(target, name).bmi
     end
 
     -- append requires flags
@@ -304,40 +294,26 @@ function make_module_buildcmds(target, batchcmds, opt)
         _append_requires_flags(target, opt.module, name, opt.cppfile, bmifile, opt)
     end
 
-    -- for cpp file we need to check after appendings the flags
-    if build == nil then
-        build = should_build(target, opt.cppfile, bmifile, {name = name, objectfile = opt.objectfile, requires = opt.module.requires})
-    end
+    -- compile if it's a named module
+    if provide or compiler_support.has_module_extension(opt.cppfile) then
+        batchcmds:mkdir(path.directory(opt.objectfile))
 
-    if build then
-        -- compile if it's a named module
-        if provide or compiler_support.has_module_extension(opt.cppfile) then
-            batchcmds:mkdir(path.directory(opt.objectfile))
-
-            local fileconfig = target:fileconfig(opt.cppfile)
-            local public = fileconfig and fileconfig.public
-            local external = fileconfig and fileconfig.external
-            local bmifile = mapped_bmi or bmifile
-            if target:is_binary() then
-                if mapped_bmi then
-                    batchcmds:show_progress(opt.progress, "${color.build.target}<%s> ${clear}${color.build.object}compiling.objectfile.$(mode) %s", target:name(), name or opt.cppfile)
-                    _compile_objectfile_step(target, bmifile, opt.cppfile, opt.objectfile, {batchcmds = batchcmds})
-                else
-                    batchcmds:show_progress(opt.progress, "${color.build.target}<%s> ${clear}${color.build.object}compiling.module.$(mode) %s", target:name(), name or opt.cppfile)
-                    _compile_one_step(target, bmifile, opt.cppfile, opt.objectfile, {std = (name == "std" or name == "std.compat"), batchcmds = batchcmds})
-                end
-            else
-                if not public and not external then
-                    batchcmds:show_progress(opt.progress, "${color.build.target}<%s> ${clear}${color.build.object}compiling.module.$(mode) %s", target:name(), name or opt.cppfile)
-                    _compile_one_step(target, bmifile, opt.cppfile, opt.objectfile, {std = (name == "std" or name == "std.compat"), batchcmds = batchcmds})
-                else
-                    batchcmds:show_progress(opt.progress, "${color.build.target}<%s> ${clear}${color.build.object}compiling.bmi.$(mode) %s", target:name(), name or opt.cppfile)
-                    _compile_bmi_step(target, bmifile, opt.cppfile, {std = (name == "std" or name == "std.compat"), batchcmds = batchcmds})
-                end
+        local fileconfig = target:fileconfig(opt.cppfile)
+        local public = fileconfig and fileconfig.public
+        local external = fileconfig and fileconfig.external
+        local bmifile = mapped_bmi or bmifile
+        local is_mapped_bmi = mapped_bmi ~= nil
+        if external and not from_moduleonly then
+            if not mapped_bmi then
+                batchcmds:show_progress(opt.progress, "${color.build.target}<%s> ${clear}${color.build.object}compiling.bmi.$(mode) %s", target:name(), name or opt.cppfile)
+                _compile_bmi_step(target, bmifile, opt.cppfile, {std = (name == "std" or name == "std.compat"), batchcmds = batchcmds})
             end
         else
-            batchcmds:rm(opt.objectfile) -- force rebuild for .cpp files
+            batchcmds:show_progress(opt.progress, "${color.build.target}<%s> ${clear}${color.build.object}compiling.module.$(mode) %s", target:name(), name or opt.cppfile)
+            _compile_one_step(target, bmifile, opt.cppfile, opt.objectfile, {std = (name == "std" or name == "std.compat"), batchcmds = batchcmds, is_mapped_bmi = is_mapped_bmi})
         end
+    else
+        batchcmds:rm(opt.objectfile) -- force rebuild for .cpp files
     end
     batchcmds:add_depfiles(opt.cppfile)
     return os.mtime(opt.objectfile)
@@ -350,7 +326,7 @@ function make_headerunit_buildjobs(target, job_name, batchjobs, headerunit, bmif
         return {
             name = job_name,
             sourcefile = headerunit.path,
-            job = batchjobs:newjob(job_name, function(index, total)
+            job = batchjobs:newjob(job_name, function(index, total, jobopt)
                 if not os.isdir(outputdir) then
                     os.mkdir(outputdir)
                 end
@@ -364,7 +340,7 @@ function make_headerunit_buildjobs(target, job_name, batchjobs, headerunit, bmif
                 local depvalues = {compinst:program(), compflags}
 
                 if opt.build then
-                    progress.show((index * 100) / total, "${color.build.target}<%s> ${clear}${color.build.object}compiling.headerunit.$(mode) %s", target:name(), headerunit.name)
+                    progress.show(jobopt.progress, "${color.build.target}<%s> ${clear}${color.build.object}compiling.headerunit.$(mode) %s", target:name(), headerunit.name)
                     _compile(target, _make_headerunitflags(target, headerunit, bmifile), headerunit.path, bmifile)
                 end
 
@@ -398,4 +374,3 @@ function get_requires(target, module)
     end
     return _requires
 end
-
