@@ -31,7 +31,7 @@ function _dependency_scanner(target)
     local cachekey = tostring(target)
     local dependency_scanner = compiler_support.memcache():get2("dependency_scanner", cachekey)
     if dependency_scanner == nil then
-        if target:has_tool("cxx", "clang", "clangxx") then
+        if target:has_tool("cxx", "clang", "clangxx", "clang_cl") then
             dependency_scanner = import("clang.dependency_scanner", {anonymous = true})
         elseif target:has_tool("cxx", "gcc", "gxx") then
             dependency_scanner = import("gcc.dependency_scanner", {anonymous = true})
