@@ -32,19 +32,26 @@ function main(toolchain)
     local bindir = toolchain:bindir()
 
     -- find cross toolchain from external envirnoment
-    local cross_toolchain = find_cross_toolchain(sdkdir, {bindir = bindir})
-    if not cross_toolchain then
-        -- find it from packages
-        for _, package in ipairs(toolchain:packages()) do
-            local installdir = package:installdir()
-            if installdir and os.isdir(installdir) then
-                cross_toolchain = find_cross_toolchain(installdir)
-                if cross_toolchain then
-                    break
+    local envs
+    local cross_toolchain
+    -- find it from packages first, because we need bind msys2 envirnoments from package.
+    for _, package in ipairs(toolchain:packages()) do
+        local installdir = package:installdir()
+        if installdir and os.isdir(installdir) then
+            cross_toolchain = find_cross_toolchain(installdir)
+            if cross_toolchain then
+                -- we need to bind msys2 shell envirnoments for calling cosmocc,
+                -- @see https://github.com/xmake-io/xmake/issues/5552
+                if is_subhost("windows") then
+                    envs = package:envs()
                 end
+                break
             end
         end
     end
+    if not cross_toolchain then
+        cross_toolchain = find_cross_toolchain(sdkdir, {bindir = bindir})
+    end
     if not cross_toolchain then
         local cosmocc = find_tool("cosmocc", {force = true})
         if cosmocc and cosmocc.program then
@@ -57,9 +64,13 @@ function main(toolchain)
         toolchain:config_set("cross", cross_toolchain.cross)
         toolchain:config_set("bindir", cross_toolchain.bindir)
         toolchain:config_set("sdkdir", cross_toolchain.sdkdir)
+        if envs then
+            toolchain:config_set("envs", envs)
+        end
         toolchain:configs_save()
     else
         raise("cosmocc toolchain not found!")
     end
     return cross_toolchain
 end
+
