@@ -62,6 +62,33 @@ toolchain("llvm")
             march = "-m64"
         elseif toolchain:is_arch("i386", "x86") then
             march = "-m32"
+        elseif toolchain:is_plat("cross") then
+            local sysroot
+            local sdkdir = toolchain:sdkdir()
+            local bindir = toolchain:bindir()
+            local cross = toolchain:cross():gsub("(.*)%-$", "%1")
+            march = "--target=" .. cross
+            if bindir and os.isexec(path.join(bindir, cross .. "-gcc" .. (is_host("windows") and ".exe" or ""))) then
+                local gcc_toolchain = path.directory(bindir)
+                toolchain:add("cxflags", "--gcc-toolchain=" .. gcc_toolchain)
+                toolchain:add("mxflags", "--gcc-toolchain=" .. gcc_toolchain)
+                toolchain:add("asflags", "--gcc-toolchain=" .. gcc_toolchain)
+                toolchain:add("ldflags", "--gcc-toolchain=" .. gcc_toolchain)
+                toolchain:add("shflags", "--gcc-toolchain=" .. gcc_toolchain)
+            end
+            if sdkdir and os.isdir(path.join(sdkdir, cross, "include")) then
+                sysroot = path.join(sdkdir, cross)
+            end
+            if sysroot then
+                if os.isdir(path.join(sysroot, "libc")) then
+                    sysroot = path.join(sysroot, "libc")
+                end
+                toolchain:add("cxflags", "--sysroot=" .. sysroot)
+                toolchain:add("mxflags", "--sysroot=" .. sysroot)
+                toolchain:add("asflags", "--sysroot=" .. sysroot)
+                toolchain:add("ldflags", "--sysroot=" .. sysroot)
+                toolchain:add("shflags", "--sysroot=" .. sysroot)
+            end
         end
         if march then
             toolchain:add("cxflags", march)
